#!/bin/csh

if ($#argv != 2) then
    echo "Usage: $0 <software> <directory>"
    echo "   <software> can be simfold, pairfold or multifold"
    echo "Examples:"
    echo "$0 simfold ../simfold"
    echo "$0 pairfold ../pairfold"
    echo "$0 multifold ../multifold" 
    exit    
endif    

set soft = $argv[1]
set dir = $argv[2]


if (!(($soft =~ "simfold") ||($soft =~ "pairfold") || ($soft =~ "multifold"))) then
    echo "Usage: $0 <software> <directory>"
    echo "   <software> can be simfold, pairfold or multifold"
    echo "Examples:"
    echo "$0 simfold ../simfold"
    echo "$0 pairfold ../pairfold"
    echo "$0 multifold ../multifold" 
    exit
endif    

if (-e "$dir") then
    echo "Cannot create dir: $dir already exists."
    exit
endif     
 
echo "Making directory $dir ..."
mkdir $dir

echo "Copying parameters ..."
cp -rf params/ $dir

echo "Copying sources ..."
mkdir $dir/src
cp -rf src/common/ $dir/src/
cp -rf src/$soft/ $dir/src/

echo "Copying header files ..."
mkdir $dir/include
cp -rf include/init.h include/constants.h include/externs.h include/globals.h include/structs.h include/$soft.h $dir/include

echo "Copying driver ..."
cp -rf $soft*.cpp $dir

echo "Copying Makefile ..."
cp -rf Makefile_$soft $dir/Makefile

echo "Done."

