/***************************************************************************
                          params.h  -  description
                             -------------------
    begin                : Sun May 8 2005
    copyright            : (C) 2005 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PARAMS_H
#define PARAMS_H

#include "constants.h"

void print_stacking_energies();
// prints the stacking energies

void print_tstacki_dangling_energies();
// prints the tstacki and dangling energies

void print_stack_dangling_energies();
// prints the stack and dangling energies

void print_stack_equation_dangling();
// prints the stack and dangling energies

void print_int22_tstacki();
// prints the int22 and tstacki energies

void print_int22();
// prints the int22 energies

void test_int11_differences();
// they are the same as turner's, within 0.05 kcal/mol

void test_int22_differences();
// there are some differences here, because in the 53-param model I didn't include all the measured internal loops

int check_stability_and_size (int k, int l, int o, int p);
// helper function, to detect which delta we need for the int22 parameters

void count_AU_penalty (int base_i, int base_j, double *counter);
// PRE:  base_i and base_j make a pair
// POST: increment counter at the appropriate position
// Mirela: Nov 23, 2003

void count_penalty_by_size (int size, char type, double *counter);
// PRE:  size is the size of the loop
//       type is HAIRP or INTER or BULGE
// POST: return the penalty by size of the loop
// Mirela: Nov 23, 2003

int structure_type_index (char type[]);
  // Mirela: Nov 23, 2003
  // Given the type as a string, return the index in string_params

void count_asymmetry_penalty (int size1, int size2, double *counter);
// PRE:  size1 and size2 are the sizes of the two free base groups
// POST: Calculate the asymmetry penalty for internal loops
//       Note that if size1 == size2, pen is 0
// Mirela: Nov 23, 2003

void save_parameters (char *filename);
  // Mirela: Dec 16, 2003
  // save all parameters in the given file

void save_parameters_in_array (int *array);
// PRE: parameters have been read
// save all parameters in the given array  
  
int create_building_block_strings ();
// Mirela: Sep 20, 2005
// For most of the building blocks, write which sequence(s) correspond to each building block
// also fill string_params
  
int create_string_params ();
  // Mirela: Nov 23, 2003
  // writes each parameter type, excluding duplicates, in a long vector, containing the names of the parameters

void save_paramtypes (char *filename);
// PRE: call create_string_params ()
// save all parameter types in the given file  
  
void count_each_structure_type (char *sequence, char *structure, double *counter, int reset);
// Mirela: Nov 23, 2003
// PRE: string_params have been filled, i.e. by calling create_string_params() or create_building_block_strings()
// GIven the sequence and the structure, returns the vector counter, with the #of elementary structures
// If reset is 1, then counter will be reset to 0 first, otherwise, the values will be added to the current value of counter


double simfold_restricted_logZ (char *sequence, char *real_structure, char *restricted, double &min_energy, double &max_energy, int &actual_num_str);

void fill_data_structures_with_new_parameters (char *filename);
  // Mirela: Dec 16, 2003
  // reads parameters from a file, and writes them in the internal data structures
  // PRE: first read the actual standard parameters, to be able to figure out which of them are
  // < INF, and maybe to also keep some old values.

void fill_data_structures_with_new_parameters_fixed_dangles (char *filename, char *dangfilename);
// reads all params from filename, except the dangling parameters, which reads from a different file
// assumes the fm363 model  

int get_info_from_file (FILE *file, char *sequence, char *real_structure, char *restricted);  
// return 1 if everything was ok
// return 0 instead of continue;
  
double compute_f (char *input_file);
// PRE:  the parameters have been read
// POST: computes the f function, which is sum_{i=1}^N (1/RT G_{i,nat,theta} + logZ)

long double compute_likelihood_exactly (char *input_file);
// created on Oct 7, 2006
// PRE:  the parameters have been read
// POST: computes the likelihood function, which is \prod_{i=1}^N (exp (1/RT G_{i,nat,theta}) / Z)
// the partition function is computed exactly.
// the restriction string is not included so far

long double compute_log_likelihood_smart (char *input_file);

void compute_gradient_f (char *input_file, double *f_gradient);
// the exhaustive way
// PRE:  the parameters have been read
// POST: compute the gradient of f, which is 1/RT sum_{i=1}^N (c_{nat}^i - sum_k c_k^i exp (-1/RT G_{i,k,theta}) / sum_k exp (-1/RT G_{i,k,theta}))

void compute_gradient_f_smart (char *input_file, double *f_gradient);
// using dynamic programming
// PRE:  the parameters have been read
// POST: compute the gradient of f, which is 1/RT sum_{i=1}^N (c_{nat}^i - sum_k c_k^i exp (-1/RT G_{i,k,theta}) / sum_k exp (-1/RT G_{i,k,theta}))

double compute_f_and_gradient_f_smart (char *input_file, double *f_gradient);
// PRE:  the parameters have been read
// POST: compute the gradient of f, which is 1/RT sum_{i=1}^N (c_{nat}^i - sum_k c_k^i exp (-1/RT G_{i,k,theta}) / sum_k exp (-1/RT G_{i,k,theta}))
// returns f
// unrestricted for now

double compute_f_and_gradient_f (char *input_file, double *f_gradient);
// PRE:  the parameters have been read
// POST: computes the f function, which is sum_{i=1}^N (1/RT G_{i,nat,theta} + logZ)
// POST: compute the gradient of f, which is 1/RT sum_{i=1}^N (c_{nat}^i - sum_k c_k^i exp (-1/RT G_{i,k,theta}) / sum_k exp (-1/RT G_{i,k,theta}))

void compute_counts_vector_LP (char *input_file, double *total_counter);
// PRE:  the parameters have been read
// POST: given p input sequence/structure pairs, compute sum_{j=1}^p (c_j, cbar_j)
//          where c_j is the counts for the known structures
//          and cbar_j is the counts for the predicted mfe structure, with the given set of parameters

void compute_counts_matrix_LP (char *input_file, int train_samples);
// PRE:  the parameters have been read
//       train_samples = # of training instances. -1 if all.
// POST: given p input sequence/structure pairs, compute and display the counts c_j - cbar_j
//          where c_j is the counts for the known structures
//          and cbar_j is the counts for the predicted mfe structure, with the given set of parameters

int restricted_compatible (char *given_restricted, char *restricted);
// assume given_restricted doesn't have dots
// return 1 if the two are compatible
// given_restricted is input variable
// restricted is input-output, and it will contain the joint restriction

int generate_structure_withbb (char *sequence, char *known_structure, char *restricted, char *turner_structure, char structures[][MAXSLEN], double *old_counts, int threshold);
// written by Mirela in Sep, 2005
// PRE: the bbseq_left, bbstr_left, bbseq_right and bbstr_right are filled
//        i.e. call num_params = create_building_block_strings();
// First try to find if the building block bb_index can exist in any structure.
//   If found, restrict that part, and call simfold_restricted
//   Return the number of structures found
//   Look for at most threshold structures
//   num_params = total number of parameters
//   true_fe = the estimated free energy of the true structure. 
//        We consider only those structures whose energies are <= true_fe
//   Update old_counts every time I consider a structure 


int generate_structure_withbb_many_thresholds (char *sequence, char structures[][MAXSLEN], double energies[], double *old_counts, int many_thresholds[], double true_fe, int num_params);
// many_thresholds is a threshold per parameter (INF of no threshols wanted)


void search_bb (char *sequence, double *old_counts, int threshold, int num_params);

#endif

