/***************************************************************************
                          p_min_folding.cpp  -  description
                             -------------------
    begin                : Thu Apr 11 2002
    copyright            : (C) 2002 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Supposing we have the energy matrixes constructed, we compute the MFE

#ifndef P_MIN_FOLDING_H
#define P_MIN_FOLDING_H


#include <stdio.h>
#include <string.h>
#include "structs.h"
#include "p_energy_matrix.h"
#include "p_multi_loop.h"


class p_min_folding
// Keeps all the valid RNA structure on a stack.
// 
{
    public:
  
        // The constructors
        p_min_folding (char *seq);
        p_min_folding (char *seq1, char* seq2, char *str);

        // The destructor/
        ~p_min_folding ();
        double simfold_slow ();
        double pairfold ();
        double pairfold_nointra ();
        void set_self_comp (int self_comp) { this->self_comp = self_comp; }
        void return_structure (char *structure) { strcpy (structure, this->structure); }        

    private:
        char* structure;
        p_hairpin_loop *H;
        p_stacked_pair *S;
        p_internal_loop *VBI;
        p_multi_loop *VM;
        p_energy_matrix *V;  
        int* int_sequence;
        int link;

        minimum_fold *f;// the minimum folding; 
        int *W;
        int nb_nucleotides;
        char* sequence;
        int self_comp;
        seq_interval *stack_interval;

        void allocate_space();
        double fold_sequence ();
        double fold_sequence_nointra ();
        void insert_node (int i, int j, char type);
        
        void backtrack (seq_interval *cur_interval);
        // PRE:  All matrixes V, VM, WM and W have been filled
        // POST: Discover the MFE path

        int compute_W_br2 (int j);
        int compute_W_br2_nointra (int j);
        // POST: The second branch of W formula.
        // This branch has to consider the AU_penalties and the dangling energies.
        
        void compute_W (int j);
        void compute_W_nointra (int j);
        // compute W(j)
        
        void print_result ();
};

#endif //P_MIN_FOLDING_H
